"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.IteratorResultTypes = exports.MethodSymbols = exports.METHODS = exports.RouterRootBase = void 0;
var path_to_regexp_1 = require("path-to-regexp");
exports.RouterRootBase = '/';
var METHODS;
(function (METHODS) {
    METHODS["connect"] = "connect";
    METHODS["delete"] = "delete";
    METHODS["get"] = "get";
    METHODS["head"] = "head";
    METHODS["options"] = "options";
    METHODS["post"] = "post";
    METHODS["put"] = "put";
    METHODS["patch"] = "patch";
    METHODS["trace"] = "trace";
})(METHODS = exports.METHODS || (exports.METHODS = {}));
exports.MethodSymbols = (_a = {},
    _a[METHODS.connect] = Symbol('connect'),
    _a[METHODS.delete] = Symbol('delete'),
    _a[METHODS.get] = Symbol('get'),
    _a[METHODS.head] = Symbol('head'),
    _a[METHODS.options] = Symbol('options'),
    _a[METHODS.post] = Symbol('post'),
    _a[METHODS.put] = Symbol('put'),
    _a[METHODS.patch] = Symbol('patch'),
    _a[METHODS.trace] = Symbol('trace'),
    _a);
var IteratorResultTypes;
(function (IteratorResultTypes) {
    IteratorResultTypes["END"] = "END";
    IteratorResultTypes["NOT_FOUND"] = "NOT_FOUND";
})(IteratorResultTypes = exports.IteratorResultTypes || (exports.IteratorResultTypes = {}));
var RouterState = /** @class */ (function () {
    function RouterState(initialState) {
        var _a;
        if (initialState === void 0) { initialState = (_a = {}, _a[exports.RouterRootBase] = {}, _a); }
        this.__last_path_visited__ = '';
        this.state = initialState;
    }
    RouterState.prototype.getRoot = function () {
        return this.state[exports.RouterRootBase];
    };
    RouterState.prototype.getSymbolMethod = function (method) {
        var symbol = exports.MethodSymbols[method.toLowerCase()];
        return symbol;
    };
    RouterState.prototype.matchExact = function (token, tree) {
        // methods uses symbols to get the keys in the tree store
        // So we'll be safe if we receive a token with "get" or "post"
        var partialTree = tree[token];
        return { path: token, tree: partialTree, matchWith: token };
    };
    /**
     * It wil try to match with a regular expresion.
     * Before using it first try to matchExact function
     * @param token
     * @param tree
     * @returns
     */
    RouterState.prototype.matchWith = function (token, tree) {
        return Object.entries(tree).reduce(function (acc, _a) {
            var _b = __read(_a, 2), path = _b[0], leef = _b[1];
            var regexp = (0, path_to_regexp_1.pathToRegexp)(path);
            var result = regexp.exec(token);
            if (result === null || result === void 0 ? void 0 : result.length)
                return __spreadArray(__spreadArray([], __read(acc), false), [{ path: path, tree: leef, matchWith: path }], false);
            return __spreadArray([], __read(acc), false);
        }, []);
    };
    RouterState.prototype.match = function (token, tree) {
        var result = this.matchExact(token, tree);
        if (result === null || result === void 0 ? void 0 : result.tree)
            return [result];
        return this.matchWith(token, tree);
    };
    RouterState.prototype.iterate = function (tokens, tree, currentPath) {
        var end = !Boolean(tokens.length);
        // Tokens is empty and there is not way to keep looping.
        // That means we found the path
        if (end) {
            // return the path
            return [{ status: IteratorResultTypes.END, tree: tree, tokens: tokens }, currentPath];
        }
        // Otherwise get the first token
        var _a = __read(tokens), currentToken = _a[0], restOfTokens = _a.slice(1);
        // try to get the tree for a given token in the current state tree
        var matchResults = this.match(currentToken, tree);
        // Formalize to an array of results
        var arrayOfMatches = Array.isArray(matchResults) ? matchResults : [matchResults];
        // A path may match with parameter paths. In any case it will be an array of matches. Then loop in-order
        for (var i = 0; i < arrayOfMatches.length; i++) {
            var _b = arrayOfMatches[i], leef = _b.tree, matchWith = _b.matchWith;
            // Iterates in order
            if (leef) {
                var _c = __read(this.iterate(restOfTokens !== null && restOfTokens !== void 0 ? restOfTokens : [], leef, "".concat(currentPath, "/").concat(matchWith)), 2), value = _c[0], path = _c[1];
                // If it reaches the end, then return the value and stop.
                if ((value === null || value === void 0 ? void 0 : value.status) === IteratorResultTypes.END) {
                    return [value, path];
                }
            }
        }
        // default case: path was not found and return the last seen leef
        return [{ status: IteratorResultTypes.NOT_FOUND, tree: tree }];
    };
    RouterState.prototype.search = function (tokens, tree) {
        var _a = __read(this.iterate(tokens, tree, ''), 2), result = _a[0], path = _a[1];
        if ((result === null || result === void 0 ? void 0 : result.status) === IteratorResultTypes.END) {
            this.__last_path_visited__ = path;
            return result.tree;
        }
        this.__last_path_visited__ = '';
    };
    /**
     * Given an index return the value of that node
     * @param index IDataStructureIndexType
     * @returns T
     */
    RouterState.prototype.get = function (index) {
        var _a = __read(index.split(RouterState.SEPARATOR), 2), method = _a[0], fullPath = _a[1];
        var tokens = fullPath.split('/').filter(Boolean);
        var partialTree = this.search(tokens, this.getRoot());
        if (!partialTree)
            return;
        var symbol = this.getSymbolMethod(method);
        var handler = partialTree[symbol];
        return handler;
    };
    /**
     * This function does the same that get, butit also returns the path of the item in the tree. Useful to get
     * the proxy path it matches with. Note the path does not contain the trailing slash
     * @param index
     * @returns [T,string] where T is the handler, and the string is the path that points to handler in the treee
     */
    RouterState.prototype.getAndReturnPath = function (index) {
        var _a;
        var item = this.get(index);
        var pathToItem = ((_a = this.__last_path_visited__) === null || _a === void 0 ? void 0 : _a.length) ? this.__last_path_visited__ : undefined;
        return [item, item ? pathToItem : undefined];
    };
    /**
     * Insert an element T at the given index
     * @param index IDataStructureIndexType index where the value will be stored
     * @param value
     */
    RouterState.prototype.insert = function (index, value) {
        var _a;
        var _b = __read(index.split(RouterState.SEPARATOR), 2), method = _b[0], fullPath = _b[1];
        var tokens = fullPath.split('/').filter(Boolean);
        var iToken = 0;
        var partialTree = this.getRoot();
        while (iToken < tokens.length) {
            var token = tokens[iToken];
            // default value
            var currentTokenTree = (_a = partialTree[token]) !== null && _a !== void 0 ? _a : {};
            // initialize or set the current
            partialTree[token] = currentTokenTree;
            // Move the tree pointer
            partialTree = currentTokenTree;
            // move the index
            iToken++;
        }
        // once we are at the leef, then add the method.
        partialTree[exports.MethodSymbols[method.toLowerCase()]] = value;
        return value;
    };
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    RouterState.prototype.remove = function (index) {
        throw new Error('Method not implemented.');
    };
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    RouterState.prototype.update = function (index) {
        throw new Error('Method not implemented.');
    };
    RouterState.SEPARATOR = ':::';
    return RouterState;
}());
exports.default = RouterState;
